# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import custom_storage_credential_create_params, custom_storage_credential_update_params
from .._types import Body, Query, Headers, NoneType, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.custom_storage_credential_create_response import CustomStorageCredentialCreateResponse
from ..types.custom_storage_credential_update_response import CustomStorageCredentialUpdateResponse
from ..types.custom_storage_credential_retrieve_response import CustomStorageCredentialRetrieveResponse

__all__ = ["CustomStorageCredentialsResource", "AsyncCustomStorageCredentialsResource"]


class CustomStorageCredentialsResource(SyncAPIResource):
    """Call Recordings operations."""

    @cached_property
    def with_raw_response(self) -> CustomStorageCredentialsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CustomStorageCredentialsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomStorageCredentialsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CustomStorageCredentialsResourceWithStreamingResponse(self)

    def create(
        self,
        connection_id: str,
        *,
        backend: Literal["gcs", "s3", "azure"],
        configuration: custom_storage_credential_create_params.Configuration,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomStorageCredentialCreateResponse:
        """
        Creates a custom storage credentials configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return self._post(
            f"/custom_storage_credentials/{connection_id}",
            body=maybe_transform(
                {
                    "backend": backend,
                    "configuration": configuration,
                },
                custom_storage_credential_create_params.CustomStorageCredentialCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomStorageCredentialCreateResponse,
        )

    def retrieve(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomStorageCredentialRetrieveResponse:
        """
        Returns the information about custom storage credentials.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return self._get(
            f"/custom_storage_credentials/{connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomStorageCredentialRetrieveResponse,
        )

    def update(
        self,
        connection_id: str,
        *,
        backend: Literal["gcs", "s3", "azure"],
        configuration: custom_storage_credential_update_params.Configuration,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomStorageCredentialUpdateResponse:
        """
        Updates a stored custom credentials configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return self._put(
            f"/custom_storage_credentials/{connection_id}",
            body=maybe_transform(
                {
                    "backend": backend,
                    "configuration": configuration,
                },
                custom_storage_credential_update_params.CustomStorageCredentialUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomStorageCredentialUpdateResponse,
        )

    def delete(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes a stored custom credentials configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/custom_storage_credentials/{connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncCustomStorageCredentialsResource(AsyncAPIResource):
    """Call Recordings operations."""

    @cached_property
    def with_raw_response(self) -> AsyncCustomStorageCredentialsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomStorageCredentialsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomStorageCredentialsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCustomStorageCredentialsResourceWithStreamingResponse(self)

    async def create(
        self,
        connection_id: str,
        *,
        backend: Literal["gcs", "s3", "azure"],
        configuration: custom_storage_credential_create_params.Configuration,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomStorageCredentialCreateResponse:
        """
        Creates a custom storage credentials configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return await self._post(
            f"/custom_storage_credentials/{connection_id}",
            body=await async_maybe_transform(
                {
                    "backend": backend,
                    "configuration": configuration,
                },
                custom_storage_credential_create_params.CustomStorageCredentialCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomStorageCredentialCreateResponse,
        )

    async def retrieve(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomStorageCredentialRetrieveResponse:
        """
        Returns the information about custom storage credentials.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return await self._get(
            f"/custom_storage_credentials/{connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomStorageCredentialRetrieveResponse,
        )

    async def update(
        self,
        connection_id: str,
        *,
        backend: Literal["gcs", "s3", "azure"],
        configuration: custom_storage_credential_update_params.Configuration,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomStorageCredentialUpdateResponse:
        """
        Updates a stored custom credentials configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return await self._put(
            f"/custom_storage_credentials/{connection_id}",
            body=await async_maybe_transform(
                {
                    "backend": backend,
                    "configuration": configuration,
                },
                custom_storage_credential_update_params.CustomStorageCredentialUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomStorageCredentialUpdateResponse,
        )

    async def delete(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes a stored custom credentials configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/custom_storage_credentials/{connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class CustomStorageCredentialsResourceWithRawResponse:
    def __init__(self, custom_storage_credentials: CustomStorageCredentialsResource) -> None:
        self._custom_storage_credentials = custom_storage_credentials

        self.create = to_raw_response_wrapper(
            custom_storage_credentials.create,
        )
        self.retrieve = to_raw_response_wrapper(
            custom_storage_credentials.retrieve,
        )
        self.update = to_raw_response_wrapper(
            custom_storage_credentials.update,
        )
        self.delete = to_raw_response_wrapper(
            custom_storage_credentials.delete,
        )


class AsyncCustomStorageCredentialsResourceWithRawResponse:
    def __init__(self, custom_storage_credentials: AsyncCustomStorageCredentialsResource) -> None:
        self._custom_storage_credentials = custom_storage_credentials

        self.create = async_to_raw_response_wrapper(
            custom_storage_credentials.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            custom_storage_credentials.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            custom_storage_credentials.update,
        )
        self.delete = async_to_raw_response_wrapper(
            custom_storage_credentials.delete,
        )


class CustomStorageCredentialsResourceWithStreamingResponse:
    def __init__(self, custom_storage_credentials: CustomStorageCredentialsResource) -> None:
        self._custom_storage_credentials = custom_storage_credentials

        self.create = to_streamed_response_wrapper(
            custom_storage_credentials.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            custom_storage_credentials.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            custom_storage_credentials.update,
        )
        self.delete = to_streamed_response_wrapper(
            custom_storage_credentials.delete,
        )


class AsyncCustomStorageCredentialsResourceWithStreamingResponse:
    def __init__(self, custom_storage_credentials: AsyncCustomStorageCredentialsResource) -> None:
        self._custom_storage_credentials = custom_storage_credentials

        self.create = async_to_streamed_response_wrapper(
            custom_storage_credentials.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            custom_storage_credentials.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            custom_storage_credentials.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            custom_storage_credentials.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["RequestRetrieveStatusHistoryParams"]


class RequestRetrieveStatusHistoryParams(TypedDict, total=False):
    page_number: Required[Annotated[int, PropertyInfo(alias="page[number]")]]

    page_size: Required[Annotated[int, PropertyInfo(alias="page[size]")]]
    """Request this many records per page.

    This value is automatically clamped if the provided value is too large.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["FaxListParams", "Filter", "FilterCreatedAt", "FilterDirection", "FilterFrom", "FilterTo"]


class FaxListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[created_at][gte], filter[created_at][gt],
    filter[created_at][lte], filter[created_at][lt], filter[direction][eq],
    filter[from][eq], filter[to][eq]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class FilterCreatedAt(TypedDict, total=False):
    """Date range filtering operations for fax creation timestamp"""

    gt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """ISO 8601 date time for filtering faxes created after that date"""

    gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """ISO 8601 date time for filtering faxes created after or on that date"""

    lt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """ISO 8601 formatted date time for filtering faxes created before that date"""

    lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """ISO 8601 formatted date time for filtering faxes created on or before that date"""


class FilterDirection(TypedDict, total=False):
    """Direction filtering operations"""

    eq: str
    """The direction, inbound or outbound, for filtering faxes sent from this account"""


class FilterFrom(TypedDict, total=False):
    """From number filtering operations"""

    eq: str
    """The phone number, in E.164 format for filtering faxes sent from this number"""


class FilterTo(TypedDict, total=False):
    """To number filtering operations"""

    eq: str
    """The phone number, in E.164 format for filtering faxes sent to this number"""


_FilterReservedKeywords = TypedDict(
    "_FilterReservedKeywords",
    {
        "from": FilterFrom,
    },
    total=False,
)


class Filter(_FilterReservedKeywords, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[created_at][gte], filter[created_at][gt], filter[created_at][lte], filter[created_at][lt], filter[direction][eq], filter[from][eq], filter[to][eq]
    """

    created_at: FilterCreatedAt
    """Date range filtering operations for fax creation timestamp"""

    direction: FilterDirection
    """Direction filtering operations"""

    to: FilterTo
    """To number filtering operations"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .metadata import (
    MetadataResource,
    AsyncMetadataResource,
    MetadataResourceWithRawResponse,
    AsyncMetadataResourceWithRawResponse,
    MetadataResourceWithStreamingResponse,
    AsyncMetadataResourceWithStreamingResponse,
)
from .session_analysis import (
    SessionAnalysisResource,
    AsyncSessionAnalysisResource,
    SessionAnalysisResourceWithRawResponse,
    AsyncSessionAnalysisResourceWithRawResponse,
    SessionAnalysisResourceWithStreamingResponse,
    AsyncSessionAnalysisResourceWithStreamingResponse,
)

__all__ = [
    "MetadataResource",
    "AsyncMetadataResource",
    "MetadataResourceWithRawResponse",
    "AsyncMetadataResourceWithRawResponse",
    "MetadataResourceWithStreamingResponse",
    "AsyncMetadataResourceWithStreamingResponse",
    "SessionAnalysisResource",
    "AsyncSessionAnalysisResource",
    "SessionAnalysisResourceWithRawResponse",
    "AsyncSessionAnalysisResourceWithRawResponse",
    "SessionAnalysisResourceWithStreamingResponse",
    "AsyncSessionAnalysisResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .sim_card_data_usage_notification import SimCardDataUsageNotification

__all__ = ["SimCardDataUsageNotificationRetrieveResponse"]


class SimCardDataUsageNotificationRetrieveResponse(BaseModel):
    data: Optional[SimCardDataUsageNotification] = None
    """The SIM card individual data usage notification information."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ManagedAccountListParams", "Filter", "FilterEmail", "FilterOrganizationName"]


class ManagedAccountListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[email][contains], filter[email][eq],
    filter[organization_name][contains], filter[organization_name][eq]
    """

    include_cancelled_accounts: bool
    """Specifies if cancelled accounts should be included in the results."""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]

    sort: Literal["created_at", "email"]
    """Specifies the sort order for results.

    By default sorting direction is ascending. To have the results sorted in
    descending order add the <code> -</code> prefix.<br/><br/> That is: <ul>

      <li>
        <code>email</code>: sorts the result by the
        <code>email</code> field in ascending order.
      </li>

      <li>
        <code>-email</code>: sorts the result by the
        <code>email</code> field in descending order.
      </li>
    </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.
    """


class FilterEmail(TypedDict, total=False):
    contains: str
    """If present, email containing the given value will be returned.

    Matching is not case-sensitive. Requires at least three characters.
    """

    eq: str
    """
    If present, only returns results with the <code>email</code> matching exactly
    the value given.
    """


class FilterOrganizationName(TypedDict, total=False):
    contains: str
    """
    If present, only returns results with the <code>organization_name</code>
    containing the given value. Matching is not case-sensitive. Requires at least
    three characters.
    """

    eq: str
    """
    If present, only returns results with the <code>organization_name</code>
    matching exactly the value given.
    """


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[email][contains], filter[email][eq], filter[organization_name][contains], filter[organization_name][eq]
    """

    email: FilterEmail

    organization_name: FilterOrganizationName



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.portouts import supporting_document_create_params
from ...types.portouts.supporting_document_list_response import SupportingDocumentListResponse
from ...types.portouts.supporting_document_create_response import SupportingDocumentCreateResponse

__all__ = ["SupportingDocumentsResource", "AsyncSupportingDocumentsResource"]


class SupportingDocumentsResource(SyncAPIResource):
    """Number portout operations"""

    @cached_property
    def with_raw_response(self) -> SupportingDocumentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SupportingDocumentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SupportingDocumentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SupportingDocumentsResourceWithStreamingResponse(self)

    def create(
        self,
        id: str,
        *,
        documents: Iterable[supporting_document_create_params.Document] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SupportingDocumentCreateResponse:
        """
        Creates a list of supporting documents on a portout request.

        Args:
          documents: List of supporting documents parameters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/portouts/{id}/supporting_documents",
            body=maybe_transform(
                {"documents": documents}, supporting_document_create_params.SupportingDocumentCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SupportingDocumentCreateResponse,
        )

    def list(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SupportingDocumentListResponse:
        """
        List every supporting documents for a portout request.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/portouts/{id}/supporting_documents",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SupportingDocumentListResponse,
        )


class AsyncSupportingDocumentsResource(AsyncAPIResource):
    """Number portout operations"""

    @cached_property
    def with_raw_response(self) -> AsyncSupportingDocumentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSupportingDocumentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSupportingDocumentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSupportingDocumentsResourceWithStreamingResponse(self)

    async def create(
        self,
        id: str,
        *,
        documents: Iterable[supporting_document_create_params.Document] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SupportingDocumentCreateResponse:
        """
        Creates a list of supporting documents on a portout request.

        Args:
          documents: List of supporting documents parameters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/portouts/{id}/supporting_documents",
            body=await async_maybe_transform(
                {"documents": documents}, supporting_document_create_params.SupportingDocumentCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SupportingDocumentCreateResponse,
        )

    async def list(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SupportingDocumentListResponse:
        """
        List every supporting documents for a portout request.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/portouts/{id}/supporting_documents",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SupportingDocumentListResponse,
        )


class SupportingDocumentsResourceWithRawResponse:
    def __init__(self, supporting_documents: SupportingDocumentsResource) -> None:
        self._supporting_documents = supporting_documents

        self.create = to_raw_response_wrapper(
            supporting_documents.create,
        )
        self.list = to_raw_response_wrapper(
            supporting_documents.list,
        )


class AsyncSupportingDocumentsResourceWithRawResponse:
    def __init__(self, supporting_documents: AsyncSupportingDocumentsResource) -> None:
        self._supporting_documents = supporting_documents

        self.create = async_to_raw_response_wrapper(
            supporting_documents.create,
        )
        self.list = async_to_raw_response_wrapper(
            supporting_documents.list,
        )


class SupportingDocumentsResourceWithStreamingResponse:
    def __init__(self, supporting_documents: SupportingDocumentsResource) -> None:
        self._supporting_documents = supporting_documents

        self.create = to_streamed_response_wrapper(
            supporting_documents.create,
        )
        self.list = to_streamed_response_wrapper(
            supporting_documents.list,
        )


class AsyncSupportingDocumentsResourceWithStreamingResponse:
    def __init__(self, supporting_documents: AsyncSupportingDocumentsResource) -> None:
        self._supporting_documents = supporting_documents

        self.create = async_to_streamed_response_wrapper(
            supporting_documents.create,
        )
        self.list = async_to_streamed_response_wrapper(
            supporting_documents.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["AltBusinessIDType"]

AltBusinessIDType: TypeAlias = Literal["NONE", "DUNS", "GIIN", "LEI"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .siprec_siprec_sid_json_params import SiprecSiprecSidJsonParams as SiprecSiprecSidJsonParams
from .siprec_siprec_sid_json_response import SiprecSiprecSidJsonResponse as SiprecSiprecSidJsonResponse
from .stream_streaming_sid_json_params import StreamStreamingSidJsonParams as StreamStreamingSidJsonParams
from .stream_streaming_sid_json_response import StreamStreamingSidJsonResponse as StreamStreamingSidJsonResponse
from .recording_recording_sid_json_params import RecordingRecordingSidJsonParams as RecordingRecordingSidJsonParams
from .recording_recording_sid_json_response import (
    RecordingRecordingSidJsonResponse as RecordingRecordingSidJsonResponse,
)
from .recordings_json_recordings_json_params import (
    RecordingsJsonRecordingsJsonParams as RecordingsJsonRecordingsJsonParams,
)
from .recordings_json_recordings_json_response import (
    RecordingsJsonRecordingsJsonResponse as RecordingsJsonRecordingsJsonResponse,
)
from .recordings_json_retrieve_recordings_json_response import (
    RecordingsJsonRetrieveRecordingsJsonResponse as RecordingsJsonRetrieveRecordingsJsonResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["PrivateWirelessGatewayStatus"]


class PrivateWirelessGatewayStatus(BaseModel):
    """The current status or failure details of the Private Wireless Gateway."""

    error_code: Optional[str] = None
    """
    This attribute is an
    [error code](https://developers.telnyx.com/development/api-fundamentals/api-errors)
    related to the failure reason.
    """

    error_description: Optional[str] = None
    """This attribute provides a human-readable explanation of why a failure happened."""

    value: Optional[Literal["provisioning", "provisioned", "failed", "decommissioning"]] = None
    """The current status or failure details of the Private Wireless Gateway. <ul>

     <li><code>provisioning</code> - the Private Wireless Gateway is being provisioned.</li>
     <li><code>provisioned</code> - the Private Wireless Gateway was provisioned and able to receive connections.</li>
     <li><code>failed</code> - the provisioning had failed for a reason and it requires an intervention.</li>
     <li><code>decommissioning</code> - the Private Wireless Gateway is being removed from the network.</li>
     </ul>
     Transitioning between the provisioning and provisioned states may take some time.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["LoaConfigurationPreview0Params", "Address", "Contact", "Logo"]


class LoaConfigurationPreview0Params(TypedDict, total=False):
    address: Required[Address]
    """The address of the company."""

    company_name: Required[str]
    """The name of the company"""

    contact: Required[Contact]
    """The contact information of the company."""

    logo: Required[Logo]
    """The logo of the LOA configuration"""

    name: Required[str]
    """The name of the LOA configuration"""


class Address(TypedDict, total=False):
    """The address of the company."""

    city: Required[str]
    """The locality of the company"""

    country_code: Required[str]
    """The country code of the company"""

    state: Required[str]
    """The administrative area of the company"""

    street_address: Required[str]
    """The street address of the company"""

    zip_code: Required[str]
    """The postal code of the company"""

    extended_address: str
    """The extended address of the company"""


class Contact(TypedDict, total=False):
    """The contact information of the company."""

    email: Required[str]
    """The email address of the contact"""

    phone_number: Required[str]
    """The phone number of the contact"""


class Logo(TypedDict, total=False):
    """The logo of the LOA configuration"""

    document_id: Required[str]
    """The document identification"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime

from ...._models import BaseModel
from .event_status import EventStatus
from .conversation_channel_type import ConversationChannelType

__all__ = ["ScheduledSMSEventResponse"]


class ScheduledSMSEventResponse(BaseModel):
    assistant_id: str

    scheduled_at_fixed_datetime: datetime

    telnyx_agent_target: str

    telnyx_conversation_channel: ConversationChannelType

    telnyx_end_user_target: str

    text: str

    conversation_id: Optional[str] = None

    conversation_metadata: Optional[Dict[str, Union[str, int, bool]]] = None

    created_at: Optional[datetime] = None

    dynamic_variables: Optional[Dict[str, str]] = None
    """A map of dynamic variable names to values.

    These variables can be referenced in the assistant's instructions and messages
    using {{variable_name}} syntax.
    """

    errors: Optional[List[str]] = None

    retry_count: Optional[int] = None

    scheduled_event_id: Optional[str] = None

    status: Optional[EventStatus] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.portouts import report_list_params, report_create_params
from ...types.portouts.portout_report import PortoutReport
from ...types.portouts.report_create_response import ReportCreateResponse
from ...types.portouts.report_retrieve_response import ReportRetrieveResponse
from ...types.portouts.export_portouts_csv_report_param import ExportPortoutsCsvReportParam

__all__ = ["ReportsResource", "AsyncReportsResource"]


class ReportsResource(SyncAPIResource):
    """Number portout operations"""

    @cached_property
    def with_raw_response(self) -> ReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ReportsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        params: ExportPortoutsCsvReportParam,
        report_type: Literal["export_portouts_csv"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ReportCreateResponse:
        """
        Generate reports about port-out operations.

        Args:
          params: The parameters for generating a port-outs CSV report.

          report_type: Identifies the type of report

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/portouts/reports",
            body=maybe_transform(
                {
                    "params": params,
                    "report_type": report_type,
                },
                report_create_params.ReportCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ReportCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ReportRetrieveResponse:
        """
        Retrieve a specific report generated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/portouts/reports/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ReportRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: report_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[PortoutReport]:
        """
        List the reports generated about port-out operations.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[report_type], filter[status]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/portouts/reports",
            page=SyncDefaultFlatPagination[PortoutReport],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    report_list_params.ReportListParams,
                ),
            ),
            model=PortoutReport,
        )


class AsyncReportsResource(AsyncAPIResource):
    """Number portout operations"""

    @cached_property
    def with_raw_response(self) -> AsyncReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncReportsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        params: ExportPortoutsCsvReportParam,
        report_type: Literal["export_portouts_csv"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ReportCreateResponse:
        """
        Generate reports about port-out operations.

        Args:
          params: The parameters for generating a port-outs CSV report.

          report_type: Identifies the type of report

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/portouts/reports",
            body=await async_maybe_transform(
                {
                    "params": params,
                    "report_type": report_type,
                },
                report_create_params.ReportCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ReportCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ReportRetrieveResponse:
        """
        Retrieve a specific report generated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/portouts/reports/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ReportRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: report_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PortoutReport, AsyncDefaultFlatPagination[PortoutReport]]:
        """
        List the reports generated about port-out operations.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[report_type], filter[status]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/portouts/reports",
            page=AsyncDefaultFlatPagination[PortoutReport],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    report_list_params.ReportListParams,
                ),
            ),
            model=PortoutReport,
        )


class ReportsResourceWithRawResponse:
    def __init__(self, reports: ReportsResource) -> None:
        self._reports = reports

        self.create = to_raw_response_wrapper(
            reports.create,
        )
        self.retrieve = to_raw_response_wrapper(
            reports.retrieve,
        )
        self.list = to_raw_response_wrapper(
            reports.list,
        )


class AsyncReportsResourceWithRawResponse:
    def __init__(self, reports: AsyncReportsResource) -> None:
        self._reports = reports

        self.create = async_to_raw_response_wrapper(
            reports.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            reports.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            reports.list,
        )


class ReportsResourceWithStreamingResponse:
    def __init__(self, reports: ReportsResource) -> None:
        self._reports = reports

        self.create = to_streamed_response_wrapper(
            reports.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            reports.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            reports.list,
        )


class AsyncReportsResourceWithStreamingResponse:
    def __init__(self, reports: AsyncReportsResource) -> None:
        self._reports = reports

        self.create = async_to_streamed_response_wrapper(
            reports.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            reports.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            reports.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .porting_phone_number_extension import PortingPhoneNumberExtension

__all__ = ["PhoneNumberExtensionCreateResponse"]


class PhoneNumberExtensionCreateResponse(BaseModel):
    data: Optional[PortingPhoneNumberExtension] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.verifications.by_phone_number import action_verify_params
from ....types.verifications.by_phone_number.verify_verification_code_response import VerifyVerificationCodeResponse

__all__ = ["ActionsResource", "AsyncActionsResource"]


class ActionsResource(SyncAPIResource):
    """Two factor authentication API"""

    @cached_property
    def with_raw_response(self) -> ActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ActionsResourceWithStreamingResponse(self)

    def verify(
        self,
        phone_number: str,
        *,
        code: str,
        verify_profile_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifyVerificationCodeResponse:
        """
        Verify verification code by phone number

        Args:
          phone_number: +E164 formatted phone number.

          code: This is the code the user submits for verification.

          verify_profile_id: The identifier of the associated Verify profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._post(
            f"/verifications/by_phone_number/{phone_number}/actions/verify",
            body=maybe_transform(
                {
                    "code": code,
                    "verify_profile_id": verify_profile_id,
                },
                action_verify_params.ActionVerifyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifyVerificationCodeResponse,
        )


class AsyncActionsResource(AsyncAPIResource):
    """Two factor authentication API"""

    @cached_property
    def with_raw_response(self) -> AsyncActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncActionsResourceWithStreamingResponse(self)

    async def verify(
        self,
        phone_number: str,
        *,
        code: str,
        verify_profile_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifyVerificationCodeResponse:
        """
        Verify verification code by phone number

        Args:
          phone_number: +E164 formatted phone number.

          code: This is the code the user submits for verification.

          verify_profile_id: The identifier of the associated Verify profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._post(
            f"/verifications/by_phone_number/{phone_number}/actions/verify",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "verify_profile_id": verify_profile_id,
                },
                action_verify_params.ActionVerifyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifyVerificationCodeResponse,
        )


class ActionsResourceWithRawResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.verify = to_raw_response_wrapper(
            actions.verify,
        )


class AsyncActionsResourceWithRawResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.verify = async_to_raw_response_wrapper(
            actions.verify,
        )


class ActionsResourceWithStreamingResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.verify = to_streamed_response_wrapper(
            actions.verify,
        )


class AsyncActionsResourceWithStreamingResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.verify = async_to_streamed_response_wrapper(
            actions.verify,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from .._models import BaseModel
from .shared.messaging_pagination_meta import MessagingPaginationMeta

__all__ = ["MessagingProfileMetricListResponse"]


class MessagingProfileMetricListResponse(BaseModel):
    data: Optional[List[Dict[str, object]]] = None

    meta: Optional[MessagingPaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .reports.pagination_meta_reporting import PaginationMetaReporting

__all__ = ["ReportListMdrsResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Id of message detail record"""

    cld: Optional[str] = None
    """The destination number for a call, or the callee"""

    cli: Optional[str] = None
    """The number associated with the person initiating the call, or the caller"""

    cost: Optional[str] = None
    """Final cost. Cost is calculated as rate \\** parts"""

    created_at: Optional[datetime] = None
    """Message sent time"""

    currency: Optional[Literal["AUD", "CAD", "EUR", "GBP", "USD"]] = None
    """Currency of the rate and cost"""

    direction: Optional[str] = None
    """Direction of message - inbound or outbound."""

    message_type: Optional[Literal["SMS", "MMS"]] = None
    """Type of message"""

    parts: Optional[float] = None
    """Number of parts this message has.

    Max number of character is 160. If message contains more characters then that it
    will be broken down in multiple parts
    """

    profile_name: Optional[str] = None
    """Configured profile name.

    New profiles can be created and configured on Telnyx portal
    """

    rate: Optional[str] = None
    """Rate applied to the message"""

    record_type: Optional[str] = None

    status: Optional[
        Literal["GW_TIMEOUT", "DELIVERED", "DLR_UNCONFIRMED", "DLR_TIMEOUT", "RECEIVED", "GW_REJECT", "FAILED"]
    ] = None
    """Message status"""


class ReportListMdrsResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMetaReporting] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["NotificationEventConditionListResponse", "Parameter"]


class Parameter(BaseModel):
    data_type: Optional[str] = None

    name: Optional[str] = None

    optional: Optional[bool] = None


class NotificationEventConditionListResponse(BaseModel):
    id: Optional[str] = None
    """A UUID."""

    allow_multiple_channels: Optional[bool] = None
    """
    Dictates whether a notification channel id needs to be provided when creating a
    notficiation setting.
    """

    associated_record_type: Optional[Literal["account", "phone_number"]] = None

    asynchronous: Optional[bool] = None
    """Dictates whether a notification setting will take effect immediately."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    description: Optional[str] = None

    enabled: Optional[bool] = None

    name: Optional[str] = None

    notification_event_id: Optional[str] = None

    parameters: Optional[List[Parameter]] = None

    supported_channels: Optional[List[str]] = None
    """Dictates the supported notification channel types that can be emitted."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import webhook_delivery_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.webhook_delivery_list_response import WebhookDeliveryListResponse
from ..types.webhook_delivery_retrieve_response import WebhookDeliveryRetrieveResponse

__all__ = ["WebhookDeliveriesResource", "AsyncWebhookDeliveriesResource"]


class WebhookDeliveriesResource(SyncAPIResource):
    """Webhooks operations"""

    @cached_property
    def with_raw_response(self) -> WebhookDeliveriesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return WebhookDeliveriesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WebhookDeliveriesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return WebhookDeliveriesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookDeliveryRetrieveResponse:
        """
        Provides webhook_delivery debug data, such as timestamps, delivery status and
        attempts.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/webhook_deliveries/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookDeliveryRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: webhook_delivery_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[WebhookDeliveryListResponse]:
        """
        Lists webhook_deliveries for the authenticated user

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[status][eq], filter[event_type], filter[webhook][contains],
              filter[attempts][contains], filter[started_at][gte], filter[started_at][lte],
              filter[finished_at][gte], filter[finished_at][lte]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/webhook_deliveries",
            page=SyncDefaultFlatPagination[WebhookDeliveryListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    webhook_delivery_list_params.WebhookDeliveryListParams,
                ),
            ),
            model=WebhookDeliveryListResponse,
        )


class AsyncWebhookDeliveriesResource(AsyncAPIResource):
    """Webhooks operations"""

    @cached_property
    def with_raw_response(self) -> AsyncWebhookDeliveriesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWebhookDeliveriesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWebhookDeliveriesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncWebhookDeliveriesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookDeliveryRetrieveResponse:
        """
        Provides webhook_delivery debug data, such as timestamps, delivery status and
        attempts.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/webhook_deliveries/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookDeliveryRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: webhook_delivery_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[WebhookDeliveryListResponse, AsyncDefaultFlatPagination[WebhookDeliveryListResponse]]:
        """
        Lists webhook_deliveries for the authenticated user

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[status][eq], filter[event_type], filter[webhook][contains],
              filter[attempts][contains], filter[started_at][gte], filter[started_at][lte],
              filter[finished_at][gte], filter[finished_at][lte]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/webhook_deliveries",
            page=AsyncDefaultFlatPagination[WebhookDeliveryListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    webhook_delivery_list_params.WebhookDeliveryListParams,
                ),
            ),
            model=WebhookDeliveryListResponse,
        )


class WebhookDeliveriesResourceWithRawResponse:
    def __init__(self, webhook_deliveries: WebhookDeliveriesResource) -> None:
        self._webhook_deliveries = webhook_deliveries

        self.retrieve = to_raw_response_wrapper(
            webhook_deliveries.retrieve,
        )
        self.list = to_raw_response_wrapper(
            webhook_deliveries.list,
        )


class AsyncWebhookDeliveriesResourceWithRawResponse:
    def __init__(self, webhook_deliveries: AsyncWebhookDeliveriesResource) -> None:
        self._webhook_deliveries = webhook_deliveries

        self.retrieve = async_to_raw_response_wrapper(
            webhook_deliveries.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            webhook_deliveries.list,
        )


class WebhookDeliveriesResourceWithStreamingResponse:
    def __init__(self, webhook_deliveries: WebhookDeliveriesResource) -> None:
        self._webhook_deliveries = webhook_deliveries

        self.retrieve = to_streamed_response_wrapper(
            webhook_deliveries.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            webhook_deliveries.list,
        )


class AsyncWebhookDeliveriesResourceWithStreamingResponse:
    def __init__(self, webhook_deliveries: AsyncWebhookDeliveriesResource) -> None:
        self._webhook_deliveries = webhook_deliveries

        self.retrieve = async_to_streamed_response_wrapper(
            webhook_deliveries.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            webhook_deliveries.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .migration_source_params import MigrationSourceParams

__all__ = ["MigrationSourceRetrieveResponse"]


class MigrationSourceRetrieveResponse(BaseModel):
    data: Optional[MigrationSourceParams] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["CallCallsResponse"]


class CallCallsResponse(BaseModel):
    from_: Optional[str] = FieldInfo(alias="from", default=None)

    status: Optional[str] = None

    to: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["NetappsLocation"]


class NetappsLocation(BaseModel):
    code: Optional[str] = None
    """Location code."""

    name: Optional[str] = None
    """Human readable name of location."""

    pop: Optional[str] = None
    """Point of presence of location."""

    region: Optional[str] = None
    """Identifies the geographical region of location."""

    site: Optional[str] = None
    """Site of location."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import (
    sim_card_data_usage_notification_list_params,
    sim_card_data_usage_notification_create_params,
    sim_card_data_usage_notification_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.sim_card_data_usage_notification import SimCardDataUsageNotification
from ..types.sim_card_data_usage_notification_create_response import SimCardDataUsageNotificationCreateResponse
from ..types.sim_card_data_usage_notification_delete_response import SimCardDataUsageNotificationDeleteResponse
from ..types.sim_card_data_usage_notification_update_response import SimCardDataUsageNotificationUpdateResponse
from ..types.sim_card_data_usage_notification_retrieve_response import SimCardDataUsageNotificationRetrieveResponse

__all__ = ["SimCardDataUsageNotificationsResource", "AsyncSimCardDataUsageNotificationsResource"]


class SimCardDataUsageNotificationsResource(SyncAPIResource):
    """SIM Cards operations"""

    @cached_property
    def with_raw_response(self) -> SimCardDataUsageNotificationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SimCardDataUsageNotificationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SimCardDataUsageNotificationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SimCardDataUsageNotificationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        sim_card_id: str,
        threshold: sim_card_data_usage_notification_create_params.Threshold,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardDataUsageNotificationCreateResponse:
        """
        Creates a new SIM card data usage notification.

        Args:
          sim_card_id: The identification UUID of the related SIM card resource.

          threshold: Data usage threshold that will trigger the notification.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/sim_card_data_usage_notifications",
            body=maybe_transform(
                {
                    "sim_card_id": sim_card_id,
                    "threshold": threshold,
                },
                sim_card_data_usage_notification_create_params.SimCardDataUsageNotificationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardDataUsageNotificationCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardDataUsageNotificationRetrieveResponse:
        """
        Get a single SIM Card Data Usage Notification.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/sim_card_data_usage_notifications/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardDataUsageNotificationRetrieveResponse,
        )

    def update(
        self,
        sim_card_data_usage_notification_id: str,
        *,
        sim_card_id: str | Omit = omit,
        threshold: sim_card_data_usage_notification_update_params.Threshold | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardDataUsageNotificationUpdateResponse:
        """
        Updates information for a SIM Card Data Usage Notification.

        Args:
          sim_card_id: The identification UUID of the related SIM card resource.

          threshold: Data usage threshold that will trigger the notification.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sim_card_data_usage_notification_id:
            raise ValueError(
                f"Expected a non-empty value for `sim_card_data_usage_notification_id` but received {sim_card_data_usage_notification_id!r}"
            )
        return self._patch(
            f"/sim_card_data_usage_notifications/{sim_card_data_usage_notification_id}",
            body=maybe_transform(
                {
                    "sim_card_id": sim_card_id,
                    "threshold": threshold,
                },
                sim_card_data_usage_notification_update_params.SimCardDataUsageNotificationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardDataUsageNotificationUpdateResponse,
        )

    def list(
        self,
        *,
        filter_sim_card_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[SimCardDataUsageNotification]:
        """Lists a paginated collection of SIM card data usage notifications.

        It enables
        exploring the collection using specific filters.

        Args:
          filter_sim_card_id: A valid SIM card ID.

          page_number: The page number to load.

          page_size: The size of the page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/sim_card_data_usage_notifications",
            page=SyncDefaultFlatPagination[SimCardDataUsageNotification],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_sim_card_id": filter_sim_card_id,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    sim_card_data_usage_notification_list_params.SimCardDataUsageNotificationListParams,
                ),
            ),
            model=SimCardDataUsageNotification,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardDataUsageNotificationDeleteResponse:
        """
        Delete the SIM Card Data Usage Notification.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/sim_card_data_usage_notifications/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardDataUsageNotificationDeleteResponse,
        )


class AsyncSimCardDataUsageNotificationsResource(AsyncAPIResource):
    """SIM Cards operations"""

    @cached_property
    def with_raw_response(self) -> AsyncSimCardDataUsageNotificationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSimCardDataUsageNotificationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSimCardDataUsageNotificationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSimCardDataUsageNotificationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        sim_card_id: str,
        threshold: sim_card_data_usage_notification_create_params.Threshold,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardDataUsageNotificationCreateResponse:
        """
        Creates a new SIM card data usage notification.

        Args:
          sim_card_id: The identification UUID of the related SIM card resource.

          threshold: Data usage threshold that will trigger the notification.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/sim_card_data_usage_notifications",
            body=await async_maybe_transform(
                {
                    "sim_card_id": sim_card_id,
                    "threshold": threshold,
                },
                sim_card_data_usage_notification_create_params.SimCardDataUsageNotificationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardDataUsageNotificationCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardDataUsageNotificationRetrieveResponse:
        """
        Get a single SIM Card Data Usage Notification.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/sim_card_data_usage_notifications/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardDataUsageNotificationRetrieveResponse,
        )

    async def update(
        self,
        sim_card_data_usage_notification_id: str,
        *,
        sim_card_id: str | Omit = omit,
        threshold: sim_card_data_usage_notification_update_params.Threshold | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardDataUsageNotificationUpdateResponse:
        """
        Updates information for a SIM Card Data Usage Notification.

        Args:
          sim_card_id: The identification UUID of the related SIM card resource.

          threshold: Data usage threshold that will trigger the notification.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sim_card_data_usage_notification_id:
            raise ValueError(
                f"Expected a non-empty value for `sim_card_data_usage_notification_id` but received {sim_card_data_usage_notification_id!r}"
            )
        return await self._patch(
            f"/sim_card_data_usage_notifications/{sim_card_data_usage_notification_id}",
            body=await async_maybe_transform(
                {
                    "sim_card_id": sim_card_id,
                    "threshold": threshold,
                },
                sim_card_data_usage_notification_update_params.SimCardDataUsageNotificationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardDataUsageNotificationUpdateResponse,
        )

    def list(
        self,
        *,
        filter_sim_card_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[SimCardDataUsageNotification, AsyncDefaultFlatPagination[SimCardDataUsageNotification]]:
        """Lists a paginated collection of SIM card data usage notifications.

        It enables
        exploring the collection using specific filters.

        Args:
          filter_sim_card_id: A valid SIM card ID.

          page_number: The page number to load.

          page_size: The size of the page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/sim_card_data_usage_notifications",
            page=AsyncDefaultFlatPagination[SimCardDataUsageNotification],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_sim_card_id": filter_sim_card_id,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    sim_card_data_usage_notification_list_params.SimCardDataUsageNotificationListParams,
                ),
            ),
            model=SimCardDataUsageNotification,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardDataUsageNotificationDeleteResponse:
        """
        Delete the SIM Card Data Usage Notification.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/sim_card_data_usage_notifications/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardDataUsageNotificationDeleteResponse,
        )


class SimCardDataUsageNotificationsResourceWithRawResponse:
    def __init__(self, sim_card_data_usage_notifications: SimCardDataUsageNotificationsResource) -> None:
        self._sim_card_data_usage_notifications = sim_card_data_usage_notifications

        self.create = to_raw_response_wrapper(
            sim_card_data_usage_notifications.create,
        )
        self.retrieve = to_raw_response_wrapper(
            sim_card_data_usage_notifications.retrieve,
        )
        self.update = to_raw_response_wrapper(
            sim_card_data_usage_notifications.update,
        )
        self.list = to_raw_response_wrapper(
            sim_card_data_usage_notifications.list,
        )
        self.delete = to_raw_response_wrapper(
            sim_card_data_usage_notifications.delete,
        )


class AsyncSimCardDataUsageNotificationsResourceWithRawResponse:
    def __init__(self, sim_card_data_usage_notifications: AsyncSimCardDataUsageNotificationsResource) -> None:
        self._sim_card_data_usage_notifications = sim_card_data_usage_notifications

        self.create = async_to_raw_response_wrapper(
            sim_card_data_usage_notifications.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            sim_card_data_usage_notifications.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            sim_card_data_usage_notifications.update,
        )
        self.list = async_to_raw_response_wrapper(
            sim_card_data_usage_notifications.list,
        )
        self.delete = async_to_raw_response_wrapper(
            sim_card_data_usage_notifications.delete,
        )


class SimCardDataUsageNotificationsResourceWithStreamingResponse:
    def __init__(self, sim_card_data_usage_notifications: SimCardDataUsageNotificationsResource) -> None:
        self._sim_card_data_usage_notifications = sim_card_data_usage_notifications

        self.create = to_streamed_response_wrapper(
            sim_card_data_usage_notifications.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            sim_card_data_usage_notifications.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            sim_card_data_usage_notifications.update,
        )
        self.list = to_streamed_response_wrapper(
            sim_card_data_usage_notifications.list,
        )
        self.delete = to_streamed_response_wrapper(
            sim_card_data_usage_notifications.delete,
        )


class AsyncSimCardDataUsageNotificationsResourceWithStreamingResponse:
    def __init__(self, sim_card_data_usage_notifications: AsyncSimCardDataUsageNotificationsResource) -> None:
        self._sim_card_data_usage_notifications = sim_card_data_usage_notifications

        self.create = async_to_streamed_response_wrapper(
            sim_card_data_usage_notifications.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            sim_card_data_usage_notifications.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            sim_card_data_usage_notifications.update,
        )
        self.list = async_to_streamed_response_wrapper(
            sim_card_data_usage_notifications.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            sim_card_data_usage_notifications.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .voice import (
    VoiceResource,
    AsyncVoiceResource,
    VoiceResourceWithRawResponse,
    AsyncVoiceResourceWithRawResponse,
    VoiceResourceWithStreamingResponse,
    AsyncVoiceResourceWithStreamingResponse,
)
from .messaging import (
    MessagingResource,
    AsyncMessagingResource,
    MessagingResourceWithRawResponse,
    AsyncMessagingResourceWithRawResponse,
    MessagingResourceWithStreamingResponse,
    AsyncMessagingResourceWithStreamingResponse,
)
from .speech_to_text import (
    SpeechToTextResource,
    AsyncSpeechToTextResource,
    SpeechToTextResourceWithRawResponse,
    AsyncSpeechToTextResourceWithRawResponse,
    SpeechToTextResourceWithStreamingResponse,
    AsyncSpeechToTextResourceWithStreamingResponse,
)
from .batch_detail_records import (
    BatchDetailRecordsResource,
    AsyncBatchDetailRecordsResource,
    BatchDetailRecordsResourceWithRawResponse,
    AsyncBatchDetailRecordsResourceWithRawResponse,
    BatchDetailRecordsResourceWithStreamingResponse,
    AsyncBatchDetailRecordsResourceWithStreamingResponse,
)

__all__ = [
    "MessagingResource",
    "AsyncMessagingResource",
    "MessagingResourceWithRawResponse",
    "AsyncMessagingResourceWithRawResponse",
    "MessagingResourceWithStreamingResponse",
    "AsyncMessagingResourceWithStreamingResponse",
    "SpeechToTextResource",
    "AsyncSpeechToTextResource",
    "SpeechToTextResourceWithRawResponse",
    "AsyncSpeechToTextResourceWithRawResponse",
    "SpeechToTextResourceWithStreamingResponse",
    "AsyncSpeechToTextResourceWithStreamingResponse",
    "VoiceResource",
    "AsyncVoiceResource",
    "VoiceResourceWithRawResponse",
    "AsyncVoiceResourceWithRawResponse",
    "VoiceResourceWithStreamingResponse",
    "AsyncVoiceResourceWithStreamingResponse",
    "BatchDetailRecordsResource",
    "AsyncBatchDetailRecordsResource",
    "BatchDetailRecordsResourceWithRawResponse",
    "AsyncBatchDetailRecordsResourceWithRawResponse",
    "BatchDetailRecordsResourceWithStreamingResponse",
    "AsyncBatchDetailRecordsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["InvoiceRetrieveParams"]


class InvoiceRetrieveParams(TypedDict, total=False):
    action: Literal["json", "link"]
    """Invoice action"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["TranscriptionEngineAzureConfigParam"]


class TranscriptionEngineAzureConfigParam(TypedDict, total=False):
    region: Required[Literal["australiaeast", "centralindia", "eastus", "northcentralus", "westeurope", "westus2"]]
    """Azure region to use for speech recognition"""

    transcription_engine: Required[Literal["Azure"]]
    """Engine identifier for Azure transcription service"""

    api_key_ref: str
    """Reference to the API key for authentication.

    See
    [integration secrets documentation](https://developers.telnyx.com/api/secrets-manager/integration-secrets/create-integration-secret)
    for details. The parameter is optional as defaults are available for some
    regions.
    """

    language: Literal[
        "af",
        "am",
        "ar",
        "bg",
        "bn",
        "bs",
        "ca",
        "cs",
        "cy",
        "da",
        "de",
        "el",
        "en",
        "es",
        "et",
        "eu",
        "fa",
        "fi",
        "fr",
        "ga",
        "gl",
        "gu",
        "he",
        "hi",
        "hr",
        "hu",
        "hy",
        "id",
        "is",
        "it",
        "ja",
        "ka",
        "kk",
        "km",
        "kn",
        "ko",
        "lo",
        "lt",
        "lv",
        "mk",
        "ml",
        "mn",
        "mr",
        "ms",
        "mt",
        "my",
        "nb",
        "ne",
        "nl",
        "pl",
        "ps",
        "pt",
        "ro",
        "ru",
        "si",
        "sk",
        "sl",
        "so",
        "sq",
        "sr",
        "sv",
        "sw",
        "ta",
        "te",
        "th",
        "tr",
        "uk",
        "ur",
        "uz",
        "vi",
        "wuu",
        "yue",
        "zh",
        "zu",
        "auto",
    ]
    """Language to use for speech recognition"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["PwgAssignedResourcesSummary"]


class PwgAssignedResourcesSummary(BaseModel):
    """
    The summary of the resource that have been assigned to the Private Wireless Gateway.
    """

    count: Optional[int] = None
    """The current count of a resource type assigned to the Private Wireless Gateway."""

    record_type: Optional[str] = None
    """The type of the resource assigned to the Private Wireless Gateway."""
